use chrono_tz::Tz;
use poise::serenity_prelude::{model::id::GuildId, ResolvedValue};
use serde::{Deserialize, Serialize};
use serde_json::Value;

use crate::{commands::reminder_cmds::create_reminder, ApplicationContext, Context, Error};

#[derive(Serialize, Deserialize)]
#[serde(tag = "command_name")]
pub enum RecordedCommand {
    Remind(RemindOptions),
}

impl RecordedCommand {
    pub fn from_context(ctx: ApplicationContext) -> Option<Self> {
        match ctx.command().identifying_name.as_str() {
            "remind" => Some(Self::Remind(RemindOptions {
                time: ctx
                    .args
                    .iter()
                    .find(|opt| opt.name == "time")
                    .map(|opt| &opt.value)
                    .map_or_else(
                        || String::new(),
                        |v| match v {
                            ResolvedValue::String(s) => s.to_string(),
                            _ => String::new(),
                        },
                    ),
                content: ctx
                    .args
                    .iter()
                    .find(|opt| opt.name == "content")
                    .map(|opt| &opt.value)
                    .map_or_else(
                        || String::new(),
                        |v| match v {
                            ResolvedValue::String(s) => s.to_string(),
                            _ => String::new(),
                        },
                    ),
                channels: None,
                interval: None,
                expires: None,
                tts: None,
                timezone: None,
            })),
            _ => None,
        }
    }

    pub async fn execute(&self, ctx: ApplicationContext<'_>) -> Result<(), Error> {
        match self {
            RecordedCommand::Remind(command_options) => {
                create_reminder(
                    Context::Application(ctx),
                    command_options.time.clone(),
                    command_options.content.clone(),
                    None,
                    None,
                    None,
                    None,
                    None,
                )
                .await
            }
        }
    }
}

#[derive(Serialize, Deserialize, Default)]
pub struct RemindOptions {
    time: String,
    content: String,
    channels: Option<String>,
    interval: Option<String>,
    expires: Option<String>,
    tts: Option<bool>,
    timezone: Option<Tz>,
}

pub struct CommandMacro {
    pub guild_id: GuildId,
    pub name: String,
    pub description: Option<String>,
    pub commands: Vec<RecordedCommand>,
}

pub async fn guild_command_macro(ctx: &Context<'_>, name: &str) -> Option<CommandMacro> {
    struct Row {
        name: String,
        description: Option<String>,
        commands: Value,
    }

    let row = sqlx::query_as!(
        Row,
        "
        SELECT m.name, m.description, m.commands
        FROM command_macro m
        INNER JOIN guilds g
            ON g.id = m.guild_id
        WHERE guild = ?
            AND m.name = ?
        ",
        ctx.guild_id().unwrap().get(),
        name
    )
    .fetch_one(&ctx.data().database)
    .await
    .ok()?;

    let command_macro = CommandMacro {
        guild_id: ctx.guild_id().unwrap(),
        name: row.name,
        description: row.description,
        commands: serde_json::from_value(row.commands).unwrap(),
    };

    Some(command_macro)
}
